<?php
// +----------------------------------------------------------------------
// | VitPHP
// +----------------------------------------------------------------------
// | 版权所有 2018~2021 石家庄萌折科技有限公司 [ http://www.vitphp.cn ]
// +----------------------------------------------------------------------
// | VitPHP是一款免费开源软件,您可以访问http://www.vitphp.cn/ 以获得更多细节。
// +----------------------------------------------------------------------

namespace app\index\controller\api;

use app\BaseController;
use think\facade\Db;
use think\facade\Session;
use vitphp\admin\storage\Storage;

/**
 * @title 前端附件管理
 * @package app\index\controller\api
 */
class AppAttachment extends BaseController
{
    protected $typePath = [
        1 => 'images',
        2 => 'music',
        3 => 'video',
    ];

    /**
     * desription 压缩图片
     * @param sting $imgsrc 图片路径
     * @param string $imgdst 压缩后保存路径
     */
    private function compressedImage($imgsrc, $imgdst, $imgwidth = 800) {
        list($width, $height, $type) = getimagesize($imgsrc);
        $new_width = $width>$imgwidth?$imgwidth:$width;//图片宽度的限制
        $new_height =$height>$imgwidth?ceil($height*$imgwidth/$width):$height;//自适应匹配图片高度
        switch ($type) {
            case 1:
                $giftype = $this->check_gifcartoon($imgsrc);
                if ($giftype) {
                    header('Content-Type:image/gif');
                    $image_wp = imagecreatetruecolor($new_width, $new_height);
                    $image = imagecreatefromgif($imgsrc);
                    imagecopyresampled($image_wp, $image, 0, 0, 0, 0, $new_width, $new_height, $width, $height);
                    //90代表的是质量、压缩图片容量大小
                    imagejpeg($image_wp, $imgdst, 90);
                    imagedestroy($image_wp);
                    imagedestroy($image);
                }
                break;
            case 2:
                header('Content-Type:image/jpeg');
                $image_wp = imagecreatetruecolor($new_width, $new_height);
                $image = imagecreatefromjpeg($imgsrc);
                imagecopyresampled($image_wp, $image, 0, 0, 0, 0, $new_width, $new_height, $width, $height);
                //90代表的是质量、压缩图片容量大小
                imagejpeg($image_wp, $imgdst, 90);
                imagedestroy($image_wp);
                imagedestroy($image);
                break;
            case 3:
                header('Content-Type:image/png');
                $image_wp = imagecreatetruecolor($new_width, $new_height);
                $image = imagecreatefrompng($imgsrc);
                imagecopyresampled($image_wp, $image, 0, 0, 0, 0, $new_width, $new_height, $width, $height);
                //90代表的是质量、压缩图片容量大小
                imagejpeg($image_wp, $imgdst, 90);
                imagedestroy($image_wp);
                imagedestroy($image);
                break;
        }
    }

    /**
     * desription 判断是否gif动画
     * @param sting image_file图片路径
     * @return boolean t 是 f 否
     */

    private function check_gifcartoon($image_file){
        $fp = fopen($image_file,'rb');
        $image_head = fread($fp,1024);
        fclose($fp);
        return preg_match("/".chr(0x21).chr(0xff).chr(0x0b).'NETSCAPE2.0'."/",$image_head)?false:true;

    }
    /**
     * 前端上传图片
     */
    public function upload($type = 1, $attaType = null)
    {
        if (!$this->request->isPost()) {
            return [
                'code' => -1,
                'msg' => '请使用post提交'
            ];
        }

        $files = $this->request->file();
        $typePath = $this->typePath[$type] ?? $this->typePath[1];
        foreach ($files as $file) {
            $extension = strtolower($file->getOriginalExtension());
            if (in_array($extension, ['sh', 'asp', 'bat', 'cmd', 'exe', 'php','html','htm'])) {
                return [
                    'code' => -1,
                    'msg' => '文件安全保护，禁止上传可执行文件！'
                ];
            }
            if ($this->imgNotSafe((string)$file)) {
                return [
                    'code' => -1,
                    'msg' => '图片未通过安全检查'
                ];
            }
            if($typePath === 'images') {
                $imgwidth = getSetting('imgwidth');
                $this->compressedImage((string)$file, (string)$file, $imgwidth);
            }
            $uploadPath = 'app' . DIRECTORY_SEPARATOR . app('http')->getName() . DIRECTORY_SEPARATOR . $typePath . DIRECTORY_SEPARATOR . date('Y') . DIRECTORY_SEPARATOR . date('m') . DIRECTORY_SEPARATOR . date('d');
            try {
                $res = Storage::instance($attaType)->upload($uploadPath, $file);
            } catch (\Exception $e) {
                return [
                    'code' => -1,
                    'msg' => $e->getMessage()
                ];
            }
            $res['name'] = $res['filename'];
            return [
                'code' => 1,
                'msg' => '上传成功',
                'data' => $res
            ];
        }
        return [
            'code' => -1,
            'msg' => '未发现文件'
        ];
    }
    /**
     * 检查图片是否安全
     * @param string $filename
     * @return boolean
     */
    private function imgNotSafe(string $filename): bool
    {
        $source = fopen($filename, 'rb');
        if (($size = filesize($filename)) > 512) {
            $hexs = bin2hex(fread($source, 512));
            fseek($source, $size - 512);
            $hexs .= bin2hex(fread($source, 512));
        } else {
            $hexs = bin2hex(fread($source, $size));
        }
        if (is_resource($source)) fclose($source);
        $bins = hex2bin($hexs);
        /* 匹配十六进制中的 <% ( ) %> 或 <? ( ) ?> 或 <script | /script> */
        foreach (['<?php', '<?php ', '<% ', '<script '] as $key) if (stripos($bins, $key) !== false) return true;
        return preg_match("/(3c25.*?28.*?29.*?253e)|(3c3f.*?28.*?29.*?3f3e)|(3C534352495054)|(2F5343524950543E)|(3C736372697074)|(2F7363726970743E)/is", $hexs);
    }
}