layui.define(['jquery', 'form', 'layer', 'element','upload','browser-md5-file'], function(exports) {
    var $ = layui.jquery;

    /**
     * 消息类
     * @constructor
     */
    function Msg() {
        this.indexs = [];
        this.shade = [0.02, '#000'];
    }

    Msg.prototype = {
        constructor: Msg,
        /**
         * 关闭指定弹出层
         * @param index 弹出层索引
         */
        close: function (index) {
            var i;
            if ((i = this.indexs.indexOf(index)) !== -1) {
                this.indexs.splice(i, 1)
            }
            layer.close(index);
        },
        /**
         * 关闭全部或指定类型弹出层
         */
        closeAll: function (flag) {
            if (flag){
                console.log(flag);
                layer.closeAll(flag);
            }else{
                layer.closeAll();
            }
        },
        /**
         * 提示窗
         * @param msg 提示消息
         * @param callback 关闭后的回调
         */
        alert: function (msg, callback) {
            var index = layer.alert(msg, {end: callback, scrollbar: false});
            return this.indexs.push(index), index;
        },
        /**
         * 成功提示框
         * @param msg 提示消息
         * @param callback 关闭后的回调
         */
        success: function (msg, callback) {
            var index = layer.msg(msg, {icon: 1, shade: this.shade, end: callback, time: 2000, shadeClose: true});
            return this.indexs.push(index), index;
        },
        /**
         * 错误提示窗
         * @param msg 提示消息
         * @param callback 关闭后的回调
         */
        error: function (msg, callback) {
            var index = layer.msg(msg, {
                icon: 2,
                shade: this.shade,
                time: 3000,
                end: callback,
                shadeClose: true
            });
            return this.indexs.push(index), index;
        },
        /**
         * 问题贴士
         * @param msg 提示消息
         * @param callback 关闭后的回调
         */
        tips: function (msg, callback) {
            var index = layer.msg(msg, {time: 3000, shade: this.shade, end: callback, shadeClose: true});
            return this.indexs.push(index), index;
        },
        /**
         * 加载装
         * @param msg 加载时的消息提示
         * @param callback 关闭后的回调
         */
        loading: function (msg, callback) {
            var index = msg ? layer.msg(msg, {
                icon: 16,
                time: 0,
                end: callback
            }) : layer.load(2, {time: 0, shade :  [0.5, '#393D49'], end: callback});
            return this.indexs.push(index), index;
        },
        /**
         * 确认对话框
         * @param msg 提示消息
         * @param yes 关闭后的回调
         */
        confirm: function (msg, yes, ele) {
            var _this = this;
            var index = layer.confirm(msg, {
                title: '操作确认',
                btn: ['确认', '取消'],
                cancel:function () {
                    vitPage.btnen(ele);
                }
            }, function () {
                yes();
            }, function () {
                _this.close(index);
                vitPage.btnen(ele);
            });
        },
        /**
         * 自动处理ajax返回数据
         * @param data ajax返回数据
         * @returns {*}
         */
        auto: function (data) {
            var _this = this;
            if (parseInt(data.code) === 1){
                this.success(data.msg, function () {
                    !!data.url ? (window.location.href = data.url) : vitPage.reload();
                });
            }else{
                this.error(data.msg, function () {
                    !!data.url && (window.location.href = data.url);
                });
            }
        }
    };
    /**
     * 表单类
     * @constructor
     */
    function Page() {
    }
    Page.prototype = {
        constructor:Page,
        /**
         * 页面重新加载
         */
        reload:function () {
            location.reload();
        },
        /**
         * get请求url并使用回调处理返回数据
         * @param url   要请求的url
         * @param callback 回调函数
         */
        get: function (url, callback, ele) {
            $.get(url).fail(function (res) {
                var message = res.responseJSON.msg || " 异常，请稍后再试";
                layui.msg.error("E_Status "+res.status+message);
                vitPage.btnen(ele);
            }).done(function (res) {
                callback(res);
            }).then(function () {
                vitPage.btnen(ele);
            });
        },
        /**
         * post请求url并使用回调处理返回数据
         * @param url 要请求的url
         * @param data 要提交的数据
         * @param callback 回调函数
         */
        post: function (url, data, callback,ele) {
            $.post(url, data).fail(function (res) {
                var message;
                if (res.responseJSON === undefined){
                    message = " 异常，请稍后再试";
                }else{
                    message = res.responseJSON.msg;
                }
                layui.msg.error("E_Status "+res.status+message);
                vitPage.btnen(ele);
            }).done(function (res) {
                callback(res);
            }).then(function () {
                vitPage.btnen(ele);
            });
        },
        /**
         * 模态框中打开新地址
         * @param url 要打开的地址
         * @param title 打开后模态框的标题
         */
        modal: function (url, title, ele) {
            var glue = (url.indexOf('?') === -1) ? "?" : "&";
            var rand = "_v="+Math.random();
            var getUrl = url + glue + rand;
            var reload = $(ele).data('reload');
            var area = $(ele).data('area') || '90%,600px';
            area = area.split(',');

            $.get(getUrl).fail(function (res) {
                var message;
                if (res.responseJSON === undefined){
                    message = " 异常，请稍后再试";
                }else{
                    message = res.responseJSON.msg;
                }
                layui.msg.error("E_Status "+res.status+message);
                vitPage.btnen(ele);
            }).done(function (data) {
                if (data.code == 0){
                    layui.msg.error(data.msg)
                }else{
                    layer.open({
                        type: 1,
                        title: title,
                        content: data,
                        area: area,
                        shade: 0.1,
                        scrollbar:true,
                        end:function () {
                            if (reload == true){
                                location.reload()
                            }
                            vitPage.btnen(ele);
                        }
                    });
                }
            }).then(function () {
                 
            });
        },
        /**
         * 弹出ifreme中打开新地址
         * @param url 要打开的地址
         * @param title 打开后模态框的标题
         */
        iframe: function (url, title, ele) {
            var glue = (url.indexOf('?') === -1) ? "?" : "&";
            var rand = "_v="+Math.random();
            var getUrl = url + glue + rand;
            var reload = $(ele).data('reload');

            layer.open({
                type: 2,
                title: title,
                content: getUrl,
                area: ['100%','600px'],
                scrollbar:true,
                end:function () {
                    if (reload == true){
                        location.reload()
                    }
                    vitPage.btnen(ele);
                }
            });
        },
        /**
         * 禁用按钮
         * @param ele
         */
        btndis:function (ele) {
            $(ele).attr('disabled','disabled');//.addClass('layui-btn-disabled');
        },
        /**
         * 启用按钮
         * @param ele
         */
        btnen:function (ele) {
            $(ele).removeAttr('disabled');//.removeClass('layui-btn-disabled');
        }
    };

    /**
     * 根对象
     * @type {{msg: Window.layui.msg}}
     */
    layui.msg = new Msg();
    let vitPage = new Page();
    layui.page = new Page();

    $(function () {
        var $body= $('body');

        /**
         * [data-get] 组件，get 请求指定地址
         * 依赖 data-get="目标地址"
         *     data-conform="提示消息"
         */
        $body.on('click','[data-get]',function (e) {
            var ele = this;
            vitPage.btndis(ele);
            var url = $(this).data('get');
            var confirm = $(this).data('confirm');
            if (confirm){
                return layui.msg.confirm(confirm, getres, ele);
            }
            getres();

            /**
             * get 请求地址并处理返回参数
             */
            function getres() {
                vitPage.get(url, function (data) {
                    layui.msg.auto(data);
                }, ele);
            }
        });
        /**
         * [data-post-batch] 组件，get 请求指定地址
         * 依赖 data-post="目标地址"
         *     data-key="参数名"
         *     data-conform="提示消息"
         */
        $body.on('click','[data-post-batch]',function (e) {
            var ele = this;
            vitPage.btndis(ele);
            var url = $(this).data('post-batch');
            var key = $(this).data('key') || 'ids';
            var param = {};
            param[key] = tableCheck.getData().join(',');
            var confirm = $(this).data('confirm');
            if (confirm){
                return layui.msg.confirm(confirm, getres, ele);
            }
            getres();

            /**
             * get 请求地址并处理返回参数
             */
            function getres() {
                vitPage.post(url,param, function (data) {
                    layui.msg.auto(data);
                }, ele);
            }
        });

        /**
         * [data-get] 组件，get 请求指定地址
         * 依赖 data-post="目标地址"
         *     data-param="post参数"
         *     data-conform="提示消息"
         */
        $body.on('click','[data-post]',function (e) {
            var ele = this;
            vitPage.btndis(ele);
            var url = $(this).data('post');
            var param = $(this).data('param');
            var confirm = $(this).data('confirm');
            if (confirm){
                return layui.msg.confirm(confirm, getres, ele);
            }
            getres();

            /**
             * get 请求地址并处理返回参数
             */
            function getres() {
                vitPage.post(url,param, function (data) {
                    layui.msg.auto(data);
                }, ele);
            }
        });

        /**
         * [data-modal] 组件，模态框打开指定地址
         * 依赖 data-modal="目标地址"
         *      data-title="弹窗标题"
         */
        $body.on('click', '[data-modal]', function (e) {
            var ele = this;
            vitPage.btndis(ele);
            var url = $(this).data('modal');
            var title = $(this).data('title')||"表单";
            var area = $(this).data('area');
            vitPage.modal(url, title, ele);
        });
        /**
         * [data-iframe] 组件，模态框打开指定地址
         * 依赖 data-iframe="目标地址"
         *      data-title="弹窗标题"
         */
        $body.on('click', '[data-iframe]', function (e) {
            var ele = this;
            vitPage.btndis(ele);
            var url = $(this).data('iframe');
            var title = $(this).data('title')||"表单";
            vitPage.iframe(url, title, ele);
        });
        /**
         * data-form 表单提交 并处理相应数据
         * 依赖 data-form="目标地址"
         * 默认post提交
         */
        $body.on('submit', '[data-form]', function (e) {
            var ele = this;
            $(ele).attr('disabled','disabled').addClass('layui-btn-disabled');
            vitPage .btndis(ele);
            e.preventDefault();
            var data = $(this).serialize();
            var url = $(this).data('form');
            vitPage.post(url, data, function (data) {
                layui.msg.auto(data);
            }, ele);
        })
        /**
         * data-upload 文件上传 并处理相应数据
         * 依赖 data-upload="目标地址"
         *      data-field="上传返回值的字段"
         *      data-type="上传类型"  images（图片）、file（所有文件）、video（视频）、audio（音频）
         * 默认post提交
         */
        $body.on('click', '[data-upload]', function (e) {
            var upload = layui.upload; //得到 upload 对象
            var _this = this;
            var fetch_btn = document.createElement('button');
            var start_btn = document.createElement('button');
            var url = $(this).data('upload');
            var field = $(this).data('field');
            var type = $(this).data('type')|'images';

            function success_callback(field,res){
                $('[name="'+field+'"]').val(res.data.src);
                $('[data-preview]').each(function (index,value) {
                    if ($(value).data('preview')===field){
                        $(value).css('max-width','130px').css('max-height','130px');
                        $(value).attr('src',res.data.src);
                    }
                });
            }
            //创建一个上传组件
            upload.render({
                elem: fetch_btn
                ,url: url
                ,choose:function (obj) {
                    layui.msg.loading();
                    obj.preview(function (index, file, result) {
                        browserMD5File(file, function (err, md5) {
                            $.get("/admin/upload/checkFile.html?hash="+md5).then(function (res) {
                                if (res.code === 0){
                                    success_callback(field,res);
                                    layui.msg.closeAll('loading')
                                }else{
                                    $(start_btn).click();
                                }
                            })
                        });
                    });
                }
                ,done: function(res, index, upload){ //上传后的回调
                    layui.msg.closeAll('loading');

                    if (res.code === 1){
                        layui.msg.error(res.msg);
                        return;
                    }
                    success_callback(field,res)
                }
                ,error: function(index, upload){ //上传后的回调

                }
                ,accept: type //允许上传的文件类型
                ,size: 1024*2 //最大允许上传的文件大小 KB
                ,acceptMime:type
                ,auto:false,
                bindAction:start_btn
            });
            $(fetch_btn).click();
        });
        /**
         * data-preview 文件预览 并 点击删除数据
         * 依赖 data-preview="目标字段"
         */
        $body.on('click', '[data-preview]', function (e) {
            var field = $(this).data('preview');
            $('[name="'+field+'"]').val('');
            $(this).attr('src','');
        });
        /**
         * img.data-tips-image 组件
         * 依赖 data-preview="弹窗宽度"
         * 本组件不依赖data-*属性绑定，而是通过data-tips-image类名绑定
         */
        $body.on('click', 'img.data-tips-image', function (e) {
            var src = e.target.src;
            var img = $(this);
            $("<img/>").attr("src", $(img).attr("src")).load(function() {
                var src = e.target.src;
                var minWidth = Math.min(this.width,480);
                var width = $(e.target).data('width') || minWidth;
                var content = '<img src="'+src+'" style="width: '+width+'px;height: auto;">';
                layer.open({
                    type: 1,
                    content: content,
                    area:[width+'px'],
                    title: false,
                    shadeClose:true,
                });
            });
        });
    });

    exports('vitphp', {});
});

//后台多页面

//JavaScript代码区域
layui.use(['element', 'layer', 'jquery', 'dropdown','vitphp'], function() {
    var element = layui.element;
    var dropdown = layui.dropdown
    var $ = layui.jquery;
    var setTabNav = [{
        id: 1,
        url: '/index/admin/main.html',
        name: '首页',
        isSelect: true
    }]
    let getTabNav = localStorage.getItem('tabNav')&&localStorage.getItem('tabNav') !='[]' ? JSON.parse(localStorage.getItem('tabNav')) : [{
        id: 1,
        url: '/index/admin/main.html',
        name: '首页',
        isSelect: true
    }];
    //定义函数 绑定增加tab，删除tab，切换tab几项事件
    var tabFunction = {
        //新增tab url 页面地址 id 对应data-id name标题
        tabAdd: function(url, id, name, isSelect) {
            element.tabAdd('tables', {
                title: name,
                content: '<iframe data-frameid="' + id + '" scrolling="auto" frameborder="0" src="' + url +
                    '" style="width:100%;height: calc(100vh - 105px);"></iframe>',
                id: id
            });
            let findindex = setTabNav.findIndex(item =>{
                return item.id == id
            })
            if (findindex) {
                setTabNav.push({
                    id: id,
                    url: url,
                    name: name,
                    isSelect: isSelect
                })
            }
            if (isSelect) {
                element.tabChange('tables', id)
            }
            localStorage.setItem('tabNav', JSON.stringify(setTabNav))
        },
        //根据id切换tab
        tabChange: function(id) {
            element.tabChange('tables', id)
        },
    }

    //左侧菜单点击事件
    $('.link-active').on('click', function() {
        var dataid = $(this);
        //判断右侧是否有tab
        if ($('.layui-tab-title li[lay-id]').length <= 0) {
            tabFunction.tabAdd(dataid.attr('lay-href'), dataid.attr('data-ids'), dataid.attr('data-title'), false);
        } else {
            //判断tab是否已经存在
            var isExist = false;
            $.each($('.layui-tab-title li[lay-id]'), function() {
                //筛选id是否存在
                if ($(this).attr('lay-id') == dataid.attr("data-ids")) {
                    isExist = true;
                }
            });
            //不存在，增加tab
            if (isExist == false) {
                tabFunction.tabAdd(dataid.attr('lay-href'), dataid.attr('data-ids'), dataid.attr('data-title'), false);
            }
        }
        //转到要打开的tab
        tabFunction.tabChange(dataid.attr('data-ids'));
    });
    // 触发选项卡切换事件
    element.on('tab(tables)', function(data) {
        var id = $(this).attr('lay-id')
        setTabNav.forEach((item, index) => {
            item.isSelect = false
            if (item.id == id) {
                item.isSelect = true
                if (item.isSelect) {
                    $('.link-active').each(function(){
                        var aa = $(this).attr('data-ids');
                        if (item.id == aa) {
                            $(this).addClass('layui-this')
                            if ($(this).attr('data-type') == 'three') {
                                $(this).parent('.layui-nav-child').parent('dd').addClass('layui-nav-itemed').parent('dl').parent('li.layui-nav-item').siblings('li.layui-nav-item').children('dl').children('dd').removeClass('layui-nav-itemed')
                            }
                            if ($(this).attr('data-type') == 'one') {
                                $(this).parent('.layui-nav-item').siblings('.layui-nav-item').removeClass('layui-this')
                            }
                        } else {
                            $(this).removeClass('layui-this')
                        }
                    })
                }
            }
        })
        localStorage.setItem('tabNav', JSON.stringify(setTabNav))
    });
    // 触发删除
    element.on('tabDelete(tables)', function(data) {
        var id = $(this).parent('li').attr('lay-id')
        setTabNav.forEach((item, index) => {
            if (item.id == id) {
                setTabNav.splice(index, 1)
            }
        })
        localStorage.setItem('tabNav', JSON.stringify(setTabNav))
    });
    // 重新加载
    if (getTabNav) {
        getTabNav.forEach((item, index) => {
            tabFunction.tabAdd(item.url, item.id, item.name, item.isSelect)
            if (item.isSelect) {
                $('.link-active').each(function(){
                    var aa = $(this).attr('data-ids');
                    if (item.id == aa) {
                        $(this).addClass('layui-this')
                        if ($(this).attr('data-type') == 'three') {
                            $(this).parent('.layui-nav-child').parent('dd').addClass('layui-nav-itemed').parent('dl').parent('li.layui-nav-item').siblings('li.layui-nav-item').children('dl').children('dd').removeClass('layui-nav-itemed')
                        }
                        if ($(this).attr('data-type') == 'one') {
                            $(this).parent('.layui-nav-item').siblings('.layui-nav-item').removeClass('layui-this')
                        }
                    } else {
                        $(this).removeClass('layui-this')
                    }
                })
            }
        })
    }
    $("#refresh").click(function() {
        let iframe = $('.layui-tab-item.layui-show').find('iframe')
        iframe.attr('src', iframe.attr('src'))
    })

    //向右移动
    var less_width=0;sum_total=0;less_sum_title=0;tab_click_list=[];
    var total_list=[];   //记录菜单滚动宽度
    $('#right').click(function () {
        var sum_title = parseFloat($('.pmdefault>.layui-tab-title').width());
        //console.log(sum_title)
        //temp_data存上一次宽度
        var temp_width=0;temp_data=0;one_total_list=0;
        var temp_total=$('.pmdefault>.layui-tab-title li[lay-id]').length;
        for (var i=sum_total;i<temp_total;i++){
            tab_click_list.push($('.pmdefault>.layui-tab-title li').eq(i).attr("lay-id"));
            temp_width = parseFloat($('.pmdefault>.layui-tab-title li').eq(i).width())+30;
            if (less_width > sum_title+less_sum_title) {
                less_width = less_width - temp_data;
                less_sum_title = less_width;
                break;
            } else {
                less_width += temp_width;
                temp_data = temp_width;
                sum_total=i;
            }
        }
        if (sum_total < temp_total-1) {
            //用列表记录菜单滚动所有宽度
            total_list.push(-less_width);
            var css = {
                "left": -less_width,
            };
            $('.layui-tab-title').css(css);
        }
    })

    //向左移动
    var left_sum_width = 0;less_total_list=0;less_one_total=2;
    $('#left').click(function () {
        var sum_title=parseFloat($('.pmdefault>.layui-tab-title').width());
        //temp_data存上一次宽度
        var temp_width=0;temp_data=0;
        var temp_total=$('.pmdefault>.layui-tab-title li[lay-id]').length;
        less_total_list=total_list.length-less_one_total;
        ceshi = -total_list[less_total_list];
        if (less_total_list >= 0) {
            console.log(less_total_list, "less_total_list");
            var css = {
                "left": total_list[less_total_list]
            };
            $('.pmdefault>.layui-tab-title').css(css);
            less_one_total += 1;
        } else if (less_total_list < 0) {
            var css = {
                "left": 0
            };
            $('.pmdefault>.layui-tab-title').css(css);
            //清除所有已赋值的变量
            less_width=0;sum_total=0;less_sum_title=0;total_list=[];
        }
    });
    //初演示
    dropdown.render({
        elem: '.close'
        ,data: [{
            title: '关闭左侧'
            ,id: 1
        },{
            title: '关闭右侧'
            ,id: 2
        },{
            title: '关闭其他'
            ,id: 3
        },{
            title: '全部关闭'
            ,id: 4
        }]
        ,click: function(obj){
            let getTabNavList = localStorage.getItem('tabNav')&&localStorage.getItem('tabNav') !='[]' ? JSON.parse(localStorage.getItem('tabNav')) : [{
                id: 1,
                url: '/index/admin/main.html',
                name: '首页',
                isSelect: true
            }];
            let total = getTabNavList.length-1
            let totals = getTabNavList
            // 触发删除
            getTabNavList.forEach((item,index) =>{
                if (obj.id == 1) {
                    let a = getTabNavList.splice(1,index)
                    console.log(a)
                    a.forEach((items) =>{
                        element.tabDelete('tables', items.id);
                    })
                } else if (obj.id ==2) {
                    if (item.isSelect) {
                        let b = getTabNavList.splice(index+1,total)
                        console.log(b)
                        b.forEach((items) =>{
                            element.tabDelete('tables', items.id);
                        })
                    }
                } else if (obj.id ==3) {
                    if (item.isSelect) {
                        let d = getTabNavList.splice(1,index-1)
                        d.forEach((items) =>{
                            element.tabDelete('tables', items.id);
                        })
                        totals.forEach((items,indexs) =>{
                            if (items.isSelect) {
                                let e = totals.splice(indexs+1,total)
                                e.forEach((items) =>{
                                    element.tabDelete('tables', items.id);
                                })
                            }
                        })
                    }
                } else if (obj.id ==4) {
                    console.log(4)
                    if (index>0) {
                        let c = getTabNavList.splice(1,total)
                        c.forEach((items) =>{
                            element.tabDelete('tables', items.id);
                        })
                    }
                }
            })
            $('.layui-tab-title').css({
                left: 0
            });
            if (getTabNavList&&getTabNavList.length == 1) {
                getTabNavList.forEach(item => {
                    item.isSelect = true
                    element.tabChange('tables', item.id)
                })
            }
            localStorage.setItem('tabNav', JSON.stringify(getTabNavList))
        }
    });

    var hrefArr = window.location.href.split("#");
    if(hrefArr[1]){
        var toLen = hrefArr[1].length;
        var shortName = hrefArr[1].substring(toLen-5,toLen);
        var toHref = shortName==='.html' ?  '/'+hrefArr[1] : '/'+hrefArr[1] + '.html';
        var toIds = 0;
        var toTitle = '';

        $('.link-active').each(function(){
            if(toHref == $(this).attr('lay-href')){
                toIds = $(this).data('ids');
                toTitle = $(this).data('title');
            }
        });

        if(toIds){
            //判断右侧是否有tab
            //判断tab是否已经存在
            var isExist = false;
            $.each($('.layui-tab-title li[lay-id]'), function() {
                //筛选id是否存在
                if ($(this).attr('lay-id') == toIds) {
                    isExist = true;
                }
            });
            //不存在，增加tab
            if (isExist == false) {
                tabFunction.tabAdd(toHref, toIds, toTitle, false);
            }
            console.log('111111111111111111111有', hrefArr[1], toIds, toTitle, toHref);
            //转到要打开的tab
            tabFunction.tabChange(toIds);
        }
    }
});