<?php

namespace vitphp\admin\platform;

use think\facade\Request;
use WeChat\Contracts\Tools;
use WeChat\Exceptions\InvalidResponseException;

class Wechat
{
    protected static $wechatSessionKey = '';
    protected static $wechatInfo = [];

    protected static $wechatConfig = [];
    protected static $isSnsapiInfo = 1;

    protected static function getConfig($appid = '', $appsecret = '', $authdomain = '', $isSnsapiInfo = 1)
    {
        self::$wechatConfig = [
            'appid'=> $appid ?: (getSetting('wx_appid')?? ''),
            'appsecret'=> $appsecret ?: (getSetting('wx_appsecret')?? ''),
            'authdomain' => $authdomain ?: (getSetting('wx_sqdomain')?? ''),
            'is_snsapi_info' => $isSnsapiInfo ?: ($wechatConfig['isSnsapiInfo'] ?? 1),
        ];
    }

    /**
     * 获取微信信息
     * @return mixed
     * @throws \WeChat\Exceptions\InvalidResponseException
     * @throws \WeChat\Exceptions\LocalCacheException
     */
    public static function getWechatInfo($appid = '', $appsecret = '', $authdomain = '', $isSnsapiInfo = 1)
    {
        global $_VIT;
        self::getConfig($appid, $appsecret, $authdomain, $isSnsapiInfo);
        self::$wechatSessionKey = md5('vit_wechat_info' . $appid . '_' . self::$wechatConfig['is_snsapi_info']);
        $wechatInfo = vit_session(self::$wechatSessionKey);
        if ($wechatInfo) {
            $_VIT['wechat_user'] = $wechatInfo;
            return $wechatInfo;
        } else {
//            dump("授权"); exit;
            $wechatInfo = self::getUserInfo();
        }

        if (!empty($wechatInfo['nickname'])) {
            $wechatInfo['nickname'] = self::removeEmojiChar($wechatInfo['nickname']);
        }

        return $wechatInfo;
    }

    protected static function removeEmojiChar($str)
    {
        $mbLen = mb_strlen($str);

        $strArr = [];
        for ($i = 0; $i < $mbLen; $i++) {
            $mbSubstr = mb_substr($str, $i, 1, 'utf-8');
            if (strlen($mbSubstr) >= 4) {
                continue;
            }
            $strArr[] = $mbSubstr;
        }

        return implode('', $strArr);
    }

    protected static function getUserInfo()
    {
        global $_VIT;
        $snsapi = self::$wechatConfig['is_snsapi_info'] == 1 ? 'snsapi_userinfo' : 'snsapi_base';
        $authdomain = self::$wechatConfig['authdomain'];
        $thisUrl = Request::domain() . Request::url();
        $config = [
            'appid' => self::$wechatConfig['appid'],
            'appsecret' => self::$wechatConfig['appsecret']
        ];
        $wechat = new \WeChat\Oauth($config);
        if (!input('code')) {
            $state = '';
            $jumpUrl = $thisUrl;
            if ($authdomain) {
                $jumpUrl = $authdomain;
                $stateKey = md5(session_id() . $_VIT['pid'] . md5(time()) . $thisUrl . time());
                $state = 'vitphp-' . $stateKey;
                cache($state, $thisUrl, 60);
            }
            $redirectUrl = $wechat->getOauthRedirect($jumpUrl, $state, $snsapi);
            redirect($redirectUrl)->send();
        } else {
//            dump("有code");exit;
//            $result = $wechat->getOauthAccessToken();
            $result = Wechat::getOauthAccessToken();

//            dump("有code");exit;
            if ($snsapi == 'snsapi_userinfo') {
                $userInfo = $wechat->getUserInfo($result['access_token'], $result['openid']);
                self::$wechatInfo = array_merge($result, $userInfo);
            } else {
                self::$wechatInfo = $result;
            }
            self::$wechatInfo['code'] = input('code');
            self::$wechatInfo['state'] = input('state');
            $_VIT['wechat_user'] = self::$wechatInfo;
            vit_session(self::$wechatSessionKey, self::$wechatInfo);
            \think\facade\Session::save();
            self::removeCode();

        }
        return self::$wechatInfo;
    }

    /**
     * 通过 code 获取 AccessToken 和 openid
     * @param string $code 授权Code值，不传则取GET参数
     * @return bool|array
     * @throws Exceptions\InvalidResponseException
     * @throws Exceptions\LocalCacheException
     */
    public static  function getOauthAccessToken($code = '')
    {
        $appid = self::$wechatConfig['appid'];
        $appsecret = self::$wechatConfig['appsecret'];
        $code = $code ? $code : (isset($_GET['code']) ? $_GET['code'] : '');
        $url = "https://api.weixin.qq.com/sns/oauth2/access_token?appid={$appid}&secret={$appsecret}&code={$code}&grant_type=authorization_code";
        return Wechat::httpGetForJson($url);
    }
    /**
     * 以GET获取接口数据并转为数组
     * @param string $url 接口地址
     * @return array
     * @throws InvalidResponseException
     * @throws \WeChat\Exceptions\LocalCacheException
     */
    public  static function httpGetForJson($url)
    {
        try {
            $result = Tools::json2arr(Tools::get($url));

            return $result;
        } catch (InvalidResponseException $exception) {
            if($exception->getCode() == '40029' || $exception->getCode() == '40163'){
                Wechat::removeCode1();
            }else{
                throw new InvalidResponseException($exception->getMessage(), $exception->getCode());
            }
        }
    }
    /**
     * 剔除code与state
     */
    protected static function removeCode1()
    {
        if (!Request::isGet() || Request::isAjax()) return;
        if (!isset($_GET['code']) && !isset($_GET['state'])) return;

        $baseUrl = Request::domain() . Request::baseUrl();
        $url = $baseUrl;

        unset($_GET['code']);
        unset($_GET['state']);
        if ($_GET) {
            $url = $baseUrl . '?' . http_build_query($_GET);
        }
        echo "<script>window.location.href='{$url}'</script>";
    }
    /**
     * 剔除code与state
     */
    protected static function removeCode()
    {
        if (!Request::isGet() || Request::isAjax()) return;
        if (!isset($_GET['code']) && !isset($_GET['state'])) return;
        if (!self::$wechatInfo) return;
        $baseUrl = Request::domain() . Request::baseUrl();
        $url = $baseUrl;

        unset($_GET['code']);
        unset($_GET['state']);
        if ($_GET) {
            $url = $baseUrl . '?' . http_build_query($_GET);
        }

        echo "<script>window.location.href='{$url}'</script>";
    }
}